% Simulation of vibration of a non-uniform tubular bell
% @ Vipul Kakkad 2/2/2012

% This code runs an N-element Chebyshev-collocated simulation of a vibrating
% non-uniform beam with given log(radius) and returns the first k
% eigenvalues = sqrt(frequencies)
function [eigenvals] = free_free_nonuniform(Npoints, log_r_func, kvalues, showplots)
% Npoints = 1 x 1 number of collocation points
% r_funct = N x 1 (or 1 x N) = r(x), radius at the collocated points
% kvalues = 1 x 1 number of eigenvalues
% showplots = 1 x 1 logical of whether plots should be shown

tic

% clear all
close all
% clc
format long g

recalculate =1;
test = 0;
N = Npoints;

interval = [-1 1];
a = min(interval);
b = max(interval);

% find chebyshev nodes xvals, differentiation matrix D
[D,xvals] = cheb(N-1);
% map to right interval
[D,xvals] = map_interval_inc(D,xvals,a,b);


if(recalculate==1)
    % indices of interior nodes
    % intr = 2:(N-1); 
    
    do_reindex = [3:(N-2), 1,2,N-1,N];
    % undo_reindex = [(N-1), 1:(N-2), N];
    r_func = exp(log_r_func);
    
    E = 1e9;
    f = (E*pi/4)*(r_func.^4);
    
    M_inv = r_func.^(-2);
      
    % higher derivative matrices
    FD2 = diag(f)*D^2;
    DFD2 = D*FD2;
    D2FD2 = D*DFD2;
    
    % other rows from D2
    % first, last row from D
      
    D_til = [
            FD2(1,:);
            DFD2(1,:);
                D2FD2(3:(N-2),:);
            DFD2(N,:);
            FD2(N,:);
            ];               
        
    D_til = diag(M_inv)*D_til;
        
    D_til = D_til(do_reindex,do_reindex);            
    
    % implement the shur complement
    Amat = D_til( 1:(N-4) , 1:(N-4) );
    Bmat = D_til( 1:(N-4) , ((N-3):N) );
    Cmat = D_til( ((N-3):N) , 1:(N-4) );
    Dmat = D_til( ((N-3):N) , ((N-3):N) );
    
%     det(Dmat)
    shur = Amat - Bmat*(Dmat\Cmat);
%     shur = Amat - Bmat*(Dmat^(-1))*Cmat;

%  shur_norm = norm(shur - shur')/norm(shur)
    
    [V,E] = eig(shur);
    
    evals = (diag(E));
    % to check ratios independent of interval
    [~,index]=sort(evals);
    evals=evals(index);
    %figure
    %plot(evals,'.')  
    
    evals=evals.^0.25;
    % sorting

    V = V(:,index);              
end
% Plotting code

if(showplots==1)

    figure;
    imagesc(abs(V)); 
    colorbar;

    col = 'crgbkm';
    figure 
    hold on
    num = kvalues+2;
    for k = 1:num
%        if(k==3)
%            close
%        end
       if(mod(k,3)==0)
           figure        
       end
        subplot(3,1,mod(k,3)+1);
        hold on
        plot(xvals(3:(N-2)),V(:,k),['.-' col(mod(k,6)+1)]);
        plot([a b],[0 0],'k');
        title(['Eigenmode #' num2str(k-2) ', Eigenvalue =' num2str(evals(k))]);
    end
end
% title('Eigenfunctions (in order R,G,B,K,Y,M,C)');
% ylabel('y');
% xlabel('x');

if(test~=0)
    figure
    hold on
    m1 = evals(test)*V(:,test);
    m2 = shur*V(:,test);
    plot(xvals(3:(N-2)),m1,'og');
    plot(xvals(3:(N-2)),m2,'.-b');
    figure
    plot(xvals(3:(N-2)),abs(m1-m2),'.-r');
end

eigenvals = evals(3:(2+kvalues));

% toc
end
% evals
